/* html{
  @page{
    size: 'a4'
  }
} */

page {
  background: white;
  display: block;
  margin: 0 auto;
  margin-bottom: 0.5cm;
  box-shadow: 0 0 0.5cm rgba(0, 0, 0, 0.5);
  height: 100%;
  width: 100%;
  padding: 24px;
}

h1 {
  font-size: 24px;
  margin-bottom: 6px;
  margin-top: 0px;
  color: #3232d4;
}
h2 {
  font-size: 18px;
  margin: 0px;
}

body {
  padding: 24px;
}

p {
  margin-bottom: 6px;
  margin-top: 0px;
}

section {
  margin-bottom: 12px;
  width: 100%;
  overflow: hidden;
}

.header {
  justify-content: space-between;
  display: flex;
  flex-direction: row;

  width: 100%;
}

.wide {
  width: 100%;
}
.half {
  width: 100%;
}
.two {
  display: flex;
}
