from .default_metrics_classification import (
    all_classification_metrics,
    minimal_classification_metrics,
    multilabel_classification,
)
from .default_metrics_regression import (
    all_regression_metrics,
    low_computation_regression_mterics,
    minimal_regression_metrics,
)
